@@ -28,6 +28,7 @@
 #include <wx/strconv.h>
 #include <wx/fontmap.h>
 #include <wx/treectrl.h>
+#include <wx/progdlg.h>
 
 #include <assert.h>
 
@@ -35,15 +36,15 @@
 #include <hhcparser.h>
 
 
-namespace {
 
 // damn wxWidgets and it's scoped ptr.
 class UCharPtr {
 public:
-	UCharPtr(unsigned char *p) : _p(p) {}
+	UCharPtr(unsigned char *p, size_t size = 0) : _p(p), _size(size) {}
 	~UCharPtr() { delete[] _p; }
 
 	unsigned char *get() { return _p; }
+	size_t size() const { return _size; }
 
 private:
 	UCharPtr(const UCharPtr&);
@@ -51,10 +52,9 @@ private:
 
 private:
 	unsigned char *_p;
+	size_t _size;
 };
 
-} // namespace
-
 
 // Big-enough buffer size for use with various routines.
 #define BUF_SIZE 4096
@@ -199,24 +199,13 @@ bool CHMFile::LoadCHM(const wxString&  archiveName)
 	LoadContextIDs();
 
 #if wxUSE_UNICODE
-	// Fix the title
-	if(_enc != wxFONTENCODING_SYSTEM) {
-		wxCSConv cv(_enc);
-		wchar_t buf2[BUF_SIZE];
-		size_t len = (_title.length() < BUF_SIZE) ? 
-			_title.length() : BUF_SIZE;
-
-		size_t ret = cv.MB2WC(buf2, _title.mb_str(), len);
-		if(ret)
-			_title = wxString(buf2, ret);
-	}
+	_title = translateEncoding(_title, _enc, wxCSConv(_enc));
 #endif
 
 	return true;
 }
 
 
-
 void CHMFile::CloseCHM()
 {
 	if(_chmFile == NULL)
@@ -232,18 +221,232 @@ void CHMFile::CloseCHM()
 }
 
 
+#define MSG_RETR_TOC _("Retrieving table of contents..")
+#define MSG_RETR_IDX _("Retrieving index..")
+#define EMPTY_INDEX _("Untitled in index")
+
+
+bool CHMFile::BinaryTOC(wxTreeCtrl *toBuild, const wxCSConv& cv)
+{
+	if(!toBuild)
+		return false;
+
+	chmUnitInfo ti_ui, ts_ui, st_ui, ut_ui, us_ui;
+	
+	if(::chm_resolve_object(_chmFile, "/#TOCIDX", &ti_ui ) != 
+	   CHM_RESOLVE_SUCCESS 
+	   || ::chm_resolve_object(_chmFile, "/#TOPICS", &ts_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#STRINGS", &st_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#URLTBL", &ut_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#URLSTR", &us_ui) != 
+	   CHM_RESOLVE_SUCCESS)
+		return false; // failed to find internal files
+
+	if(ti_ui.length < 4) // just make sure
+		return false;
+
+	UCharPtr  topidx(new unsigned char[ti_ui.length], ti_ui.length);
+	UCharPtr  topics(new unsigned char[ts_ui.length], ts_ui.length);
+	UCharPtr strings(new unsigned char[st_ui.length], st_ui.length);
+	UCharPtr  urltbl(new unsigned char[ut_ui.length], ut_ui.length);
+	UCharPtr  urlstr(new unsigned char[us_ui.length], us_ui.length);
+	
+	if(::chm_retrieve_object(_chmFile, &ti_ui, topidx.get(), 
+				 0, ti_ui.length) != (size_t)ti_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &ts_ui, topics.get(), 
+				 0, ts_ui.length) != (size_t)ts_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &st_ui, strings.get(), 
+				 0, st_ui.length) != (size_t)st_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &ut_ui, urltbl.get(), 0, 
+				 ut_ui.length) != (size_t)ut_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &us_ui, urlstr.get(), 0, 
+				 us_ui.length) != (size_t)us_ui.length)
+		return false;
+
+	u_int32_t off = UINT32ARRAY(topidx.get());
+	RecurseLoadBTOC(topidx, topics, strings, urltbl, urlstr, off, 
+			cv, toBuild, 1);
+
+	return true;
+}
+
+
+void CHMFile::RecurseLoadBTOC(UCharPtr& topidx, UCharPtr& topics,
+			      UCharPtr& strings, UCharPtr& urltbl,
+			      UCharPtr& urlstr, u_int32_t offset,
+			      const wxCSConv& cv, 
+			      wxTreeCtrl *toBuild, int level)
+{
+	while(offset) {
+		if(topidx.size() < offset + 20)
+			return;
+
+		u_int32_t flags = UINT32ARRAY(topidx.get() + offset + 4);
+		u_int32_t index = UINT32ARRAY(topidx.get() + offset + 8);
+
+		if((flags & 0x4) || (flags & 0x8)) { // book or local
+			if(!GetItem(topics, strings, urltbl, urlstr, index,
+				    toBuild, NULL, wxEmptyString, cv, level, 
+				    (flags & 0x8) == 0))
+				return;
+		}
+
+		if(flags & 0x4) { // book
+
+			if(topidx.size() < offset + 24)
+				return;
+
+			u_int32_t child = UINT32ARRAY(topidx.get() + offset
+						      + 20);
+			if(child) {
+				RecurseLoadBTOC(topidx, topics, strings, 
+						urltbl, urlstr, child,
+						cv, toBuild, level + 1); 
+			}
+		}
+
+		offset = UINT32ARRAY(topidx.get() + offset + 0x10);
+	}
+}
+
+
+bool CHMFile::GetItem(UCharPtr& topics, UCharPtr& strings, UCharPtr& urltbl,
+		      UCharPtr& urlstr, u_int32_t index, 
+		      wxTreeCtrl *tree, CHMListCtrl* list, 
+		      const wxString& idxName, 
+		      const wxCSConv& UNICODE_PARAM(cv),
+		      int level, bool local)
+{
+	static wxTreeItemId parents[TREE_BUF_SIZE];
+
+	static int calls = 0;
+	static const int yieldTime = 256;
+
+	++calls;
+	if(calls % yieldTime) {
+		calls = 0;
+		wxYield();
+	}
+
+	if(tree)
+		parents[0] = tree->GetRootItem();
+
+	std::string name, value;
+
+	if(local) {
+		if(strings.size() < index + 1)
+			return false;
+		name = (char *)(strings.get() + index);
+
+	} else {
+
+		if(topics.size() < (index * 16) + 12)
+			return false;
+
+		u_int32_t offset = UINT32ARRAY(topics.get()+ (index * 16) + 4);
+		long test = (long)offset;
+
+		if(strings.size() < offset + 1)
+			return false;
+
+		if(test == -1)
+			return false;
+
+		if(!list)
+			name = (char *)(strings.get() + offset);
+
+		// #URLTBL index
+		offset = UINT32ARRAY(topics.get() + (index * 16) + 8);
+	
+		if(urltbl.size() < offset + 12)
+			return false;
+		
+		offset = UINT32ARRAY(urltbl.get() + offset + 8);
+		
+		if(urlstr.size() < offset)
+			return false;
+
+		value = (char *)(urlstr.get() + offset + 8);
+	}
+
+	if(!value.empty() && value[0] != '/')
+		value = std::string("/") + value;
+	
+	wxString tname;
+	wxString tvalue = CURRENT_CHAR_STRING(value.c_str());
+
+	if(tree && !name.empty()) {
+		int parentIndex = level ? level - 1 : 0;
+
+		tname = translateEncoding(CURRENT_CHAR_STRING(name.c_str()), 
+					  _enc, cv);
+		parents[level] = 
+			tree->AppendItem(parents[parentIndex], tname, 2, 2,
+					    new URLTreeItem(tvalue));
+		if(level) {
+			if(tree->GetItemImage(parents[parentIndex]) != 0) {
+				tree->SetItemImage(parents[parentIndex], 0,
+						   wxTreeItemIcon_Normal);
+				
+				tree->SetItemImage(parents[parentIndex], 0,
+						   wxTreeItemIcon_Selected);
+				
+				tree->SetItemImage(parents[parentIndex], 1,
+						   wxTreeItemIcon_Expanded);
+			}
+		}
+	}
+
+	if(list) {
+		tname = translateEncoding(idxName, _enc, cv);
+		if(!value.empty()) {
+			if(tname.IsEmpty())
+				tname = EMPTY_INDEX;
+		}
+
+		list->AddPairItem(tname, tvalue);
+	}
+	
+	return true;
+}
+
+
+
 bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 {
 	chmUnitInfo ui;
 	char buffer[BUF_SIZE];
 	size_t ret = BUF_SIZE - 1, curr = 0;
-
-	if(!toBuild)
+       
+	if(!toBuild) 
 		return false;
 
-	if(_topicsFile.IsEmpty() || !ResolveObject(_topicsFile, &ui))
+	wxCSConv cv(_enc);
+	
+	toBuild->Freeze();
+	bool btoc = BinaryTOC(toBuild, cv);
+	toBuild->Thaw();
+
+	if(btoc)
+		return true;
+
+	// Fall back to parsing the HTML TOC file, if that's in the archive
+	if(_topicsFile.IsEmpty() || !ResolveObject(_topicsFile, &ui)) 
 		return false;
 
+	toBuild->Freeze();
+
 	buffer[0] = '\0';
 
 	HHCParser p(_enc, toBuild, NULL);
@@ -258,10 +461,179 @@ bool CHMFile::GetTopicsTree(wxTreeCtrl *toBuild)
 
 	} while(ret == BUF_SIZE - 1);
 
+	toBuild->Thaw();
+
 	return true;
 }
 
 
+bool CHMFile::ConvertFromUnicode(std::string& value, unsigned char* buffer,
+				 size_t bufferLength)
+{
+	size_t offset = 0;
+	u_int16_t elem = 1;
+	value = "";
+
+	while(bufferLength >= offset + sizeof(elem)) {
+		elem = UINT16ARRAY(buffer + offset);
+
+		if(elem != 0) {
+			value += *((char *)(buffer + offset));
+		} else {
+			return true;
+		}
+
+		offset += sizeof(elem);
+	}
+
+	return false;
+}
+
+
+// This function is too long: prime candidate for refactoring someday
+bool CHMFile::BinaryIndex(CHMListCtrl* toBuild, const wxCSConv& cv)
+{
+	if(!toBuild)
+		return false;
+
+	chmUnitInfo bt_ui, ts_ui, st_ui, ut_ui, us_ui;
+	unsigned long items = 0;
+		
+	if(::chm_resolve_object(_chmFile, "/$WWKeywordLinks/BTree",
+				&bt_ui ) != CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#TOPICS", &ts_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#STRINGS", &st_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#URLTBL", &ut_ui) != 
+	   CHM_RESOLVE_SUCCESS
+	   || ::chm_resolve_object(_chmFile, "/#URLSTR", &us_ui) != 
+	   CHM_RESOLVE_SUCCESS)
+		return false; // failed to find internal files
+
+	UCharPtr  btree(new unsigned char[bt_ui.length], bt_ui.length);
+	UCharPtr  topics(new unsigned char[ts_ui.length], ts_ui.length);
+	UCharPtr strings(new unsigned char[st_ui.length], st_ui.length);
+	UCharPtr  urltbl(new unsigned char[ut_ui.length], ut_ui.length);
+	UCharPtr  urlstr(new unsigned char[us_ui.length], us_ui.length);
+
+	if(::chm_retrieve_object(_chmFile, &bt_ui, btree.get(), 
+				 0, bt_ui.length) != (size_t)bt_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &ts_ui, topics.get(), 
+				 0, ts_ui.length) != (size_t)ts_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &st_ui, strings.get(), 
+				 0, st_ui.length) != (size_t)st_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &ut_ui, urltbl.get(), 0, 
+				 ut_ui.length) != (size_t)ut_ui.length)
+		return false;
+
+	if(::chm_retrieve_object(_chmFile, &us_ui, urlstr.get(), 0, 
+				 us_ui.length) != (size_t)us_ui.length)
+		return false;
+
+
+	if(bt_ui.length < 0x4c + 12)
+		return false;
+
+	unsigned long offset = 0x4c;
+	int32_t next = -1;
+	int16_t freeSpace, spaceLeft;
+	const short blockSize = 2048;
+
+	do {
+		if(bt_ui.length < offset + 12)
+			return (items != 0); // end of buffer
+
+		freeSpace = UINT16ARRAY(btree.get() + offset);
+		next = INT32ARRAY(btree.get() + offset + 8);
+		spaceLeft = blockSize - 12;
+		offset += 12;
+
+		while(spaceLeft > freeSpace) {
+
+			u_int16_t tmp = 0;
+			wxString name;
+
+			do { // accumulate the index name
+				if(bt_ui.length < offset 
+				   + sizeof(u_int16_t))
+					return items != 0;
+				
+				tmp = UINT16ARRAY(btree.get() 
+						  + offset);
+				offset += sizeof(u_int16_t);
+				spaceLeft -= sizeof(u_int16_t);
+
+				name.Append(charForCode(tmp, cv, true));
+				
+			} while(tmp != 0);
+
+			if(bt_ui.length < offset + 16)
+				return (items != 0);
+			
+			u_int16_t seeAlso = UINT16ARRAY(btree.get() + offset);
+			u_int32_t numTopics = UINT32ARRAY(btree.get() + offset
+							  + 0xc);
+			offset += 16;
+			spaceLeft -= 16;
+
+			if(seeAlso) {
+
+				// TODO: something about this duplicated code
+				do { // get over the Unicode string
+					if(bt_ui.length < offset 
+					   + sizeof(u_int16_t))
+						return items != 0;
+
+					tmp = UINT16ARRAY(btree.get() 
+							  + offset);
+					offset += sizeof(u_int16_t);
+					spaceLeft -= sizeof(u_int16_t);
+					
+				} while(tmp != 0);
+			} else {
+
+				for(u_int32_t i = 0; i < numTopics
+					    && spaceLeft > freeSpace; ++i) {
+					if(bt_ui.length < offset
+					   + sizeof(u_int32_t))
+						return (items != 0);
+
+					u_int32_t index = 
+						UINT32ARRAY(btree.get() 
+							    + offset);
+
+					GetItem(topics, strings, urltbl,
+						urlstr, index, NULL, 
+						toBuild, name, cv, 0, false);
+					++items;
+
+					offset += sizeof(u_int32_t);
+					spaceLeft -= sizeof(u_int32_t);
+				}
+			}
+
+			if(bt_ui.length < offset + 8)
+				return (items != 0);
+
+			offset += 8;
+			spaceLeft -= 8;
+		}
+		
+		offset += spaceLeft;
+
+	} while(next != -1);
+
+	return (items != 0);
+}
+
+
 bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 {
 	chmUnitInfo ui;
@@ -271,9 +643,20 @@ bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 	if(!toBuild)
 		return false;
 
+	wxCSConv cv(_enc);
+	
+	toBuild->Freeze();
+	bool bindex = BinaryIndex(toBuild, cv);
+	toBuild->Thaw();
+       
+	if(bindex)
+		return true;
+
 	if(_topicsFile.IsEmpty() || !ResolveObject(_indexFile, &ui))
 		return false;
 
+	toBuild->Freeze();
+
 	buffer[0] = '\0';
 
 	HHCParser p(_enc, NULL, toBuild);
@@ -287,7 +670,8 @@ bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 		curr += ret;
 
 	} while(ret == BUF_SIZE - 1);
-
+	
+	toBuild->Thaw();
 	return true;
 }
 
@@ -552,7 +936,6 @@ bool CHMFile::GetArchiveInfo()
 }
 
 
-inline 
 u_int32_t CHMFile::GetLeafNodeOffset(const wxString& text,
 				     u_int32_t initialOffset,
 				     u_int32_t buffSize,
@@ -617,7 +1000,6 @@ u_int32_t CHMFile::GetLeafNodeOffset(const wxString& text,
 }
 
 
-inline 
 bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 			 u_int32_t wlc_offset, unsigned char ds,
 			 unsigned char dr, unsigned char cs,
@@ -666,7 +1048,7 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 
 		if(::chm_retrieve_object(_chmFile, uistrings, combuf, 
 					 stroff, COMMON_BUF_LEN - 1) == 0) {
-			topic = wxString(_("Untitled in index"));
+			topic = EMPTY_INDEX;
 
 		} else {
 			combuf[COMMON_BUF_LEN - 1] = 0;
@@ -720,7 +1102,6 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 }
 
 
-inline
 bool CHMFile::InfoFromWindows()
 {
 #define WIN_HEADER_LEN 0x08
@@ -781,7 +1162,7 @@ bool CHMFile::InfoFromWindows()
 
 			if(size && off_title)
 				_title = CURRENT_CHAR_STRING(buffer + 
-							     off_title % 4096);
+						     off_title % 4096);
 			
 			if(factor != off_home / 4096) {
 				factor = off_home / 4096;		
@@ -829,7 +1210,6 @@ bool CHMFile::InfoFromWindows()
 }
 
 
-inline
 bool CHMFile::InfoFromSystem()
 {
 	unsigned char buffer[BUF_SIZE];
