@@ -29,11 +29,14 @@
 #include <wx/hashmap.h>
 #include <wx/font.h>
 #include <wx/string.h>
+#include <string>
 
 
 // Forward declarations.
 class wxTreeCtrl;
 class CHMListCtrl;
+class UCharPtr;
+class wxCSConv;
 
 
 //! Declares a class called CHMSearchResults - <string, string> hashmap.
@@ -150,6 +153,7 @@ public:
 	 */
 	bool GetTopicsTree(wxTreeCtrl *toBuild);
 
+
 	/*!
 	  \brief Attempts to fill a CHMListCtrl by parsing the index
 	  file.
@@ -261,6 +265,29 @@ private:
 	//! Looks up as much information as possible from #SYSTEM.
 	bool InfoFromSystem();
 
+	//! Load binary TOC (if available)
+	bool BinaryTOC(wxTreeCtrl *toBuild, const wxCSConv& cv);
+
+	//! Try to recursively load the binary topics tree
+	void RecurseLoadBTOC(UCharPtr& topidx, UCharPtr& topics,
+			     UCharPtr& strings, UCharPtr& urltbl,
+			     UCharPtr& urlstr, u_int32_t offset, 
+			     const wxCSConv& cv,
+			     wxTreeCtrl *toBuild, int level);
+
+	//! Retrieve the data (name/URL) for a single entry (TOC or index)
+	bool GetItem(UCharPtr& topics, UCharPtr& strings, UCharPtr& urltbl,
+		     UCharPtr& urlstr, u_int32_t index, wxTreeCtrl *toBuild,
+		     CHMListCtrl* toBuild, const wxString& idxName,
+		     const wxCSConv& cv, int level, bool local);
+
+	//! Get the binary index (if available)
+	bool BinaryIndex(CHMListCtrl* toBuild, const wxCSConv& cv);
+
+	//! Get a regular string from a generic buffer of length bufferLength
+	bool ConvertFromUnicode(std::string& value, unsigned char* buffer,
+				size_t bufferLength);
+
 private:
 	chmFile* _chmFile;
 	wxString _filename;
