@@ -37,7 +37,11 @@
 #include <wx/filesys.h>
 #include <wx/mimetype.h>
 #include <wx/imaglist.h>
+#include <wx/bitmap.h>
 #include <wx/fs_mem.h>
+#include <wx/utils.h>
+
+#include <wx/busyinfo.h>
 
 
 #define OPEN_HELP _("Open a CHM book.")
@@ -62,8 +66,8 @@ const wxChar *greeting = wxT(
 	"Hello, and welcome to <B>xCHM</B>, the UNIX CHM viewer."
 	"<br><br><B>xCHM</B> has been written by Razvan Cojocaru "
 	"(razvanco@gmx.net). It is licensed under the <TT>GPL</TT>.<br>"
-	"<B>xCHM</B> is based on Jed Wing's <a href=\"http://66.93.236.84/"
-	"~jedwin/projects/chmlib/\">CHMLIB</a> and <a href=\"http://www."
+	"<B>xCHM</B> is based on Jed Wing's <a href=\"http://www.jedrea.com/"
+	"chmlib/\">CHMLIB</a> and <a href=\"http://www."
 	"wxwidgets.org\">wxWidgets</a>.</td></tr></table>"
 	"<br>If you'd like to know more"
 	" about CHM, you could check out <a href=\"http://www.speakeasy."
@@ -75,8 +79,14 @@ const wxChar *greeting = wxT(
 	" Portions of the fast index search are modified versions of"
 	" Pabs' <TT>GPL</TT>d <TT>chmdeco</TT> code.<br><br>If"
 	" you'd like to use the code in your own stuff please figure"
-	" <TT>GPL</TT> out first. Far too many people think <TT>GPL</TT> is"
-	" bad out of utter ignorance.<br><br>Tips:<br><ul><li>"
+	" <TT>GPL</TT> out first. Far too many people think the <TT>GPL</TT> "
+	"is bad out of utter ignorance."
+#if !defined(wxUSE_UNICODE) || !wxUSE_UNICODE
+	"<br><br><B>WARNING</B>: your <B>xCHM</B> binary is linked against a "
+	"non-Unicode version of wxWidgets! While it will work with most "
+	"CHMs, it might not properly display special character languages."
+#endif
+	"<br><br>Tips:<br><ul><li>"
 	"The global search is an \'AND\' search, i.e. searching for"
 	" \'word1 word2\' (quotes not included) will find all the pages"
 	" that contain both word1 and word2, in whatever order.</li>"
@@ -152,7 +162,8 @@ CHMFrame::CHMFrame(const wxString& title, const wxString& booksDir,
 	CreateStatusBar();
 	SetStatusText(_("Ready."));
 
-	wxMemoryFSHandler::AddFile(wxT("logo.xpm"), wxBITMAP(logo), 
+	wxBitmap bitmap(logo_xpm);
+	wxMemoryFSHandler::AddFile(wxT("logo.xpm"), bitmap, //wxBITMAP(logo), 
 				   wxBITMAP_TYPE_XPM);
 	wxMemoryFSHandler::AddFile(wxT("about.html"), greeting);
 	wxMemoryFSHandler::AddFile(wxT("error.html"), error_page);
@@ -239,7 +250,7 @@ void CHMFrame::OnChangeFonts(wxCommandEvent& WXUNUSED(event))
 		enu.EnumerateFacenames();
 		_normalFonts = new wxArrayString;
 
-#if wxMAJOR_VERSION == 2 && wxMINOR_VERSION > 7
+#if wxMAJOR_VERSION == 2 && wxMINOR_VERSION >= 7
 		*_normalFonts = enu.GetFacenames();
 #else
 		*_normalFonts = *enu.GetFacenames();
@@ -252,7 +263,7 @@ void CHMFrame::OnChangeFonts(wxCommandEvent& WXUNUSED(event))
 		enu.EnumerateFacenames(wxFONTENCODING_SYSTEM, TRUE);
 		_fixedFonts = new wxArrayString;
 
-#if wxMAJOR_VERSION == 2 && wxMINOR_VERSION > 7
+#if wxMAJOR_VERSION == 2 && wxMINOR_VERSION >= 7
 		*_fixedFonts = enu.GetFacenames();
 #else
 		*_fixedFonts = *enu.GetFacenames();
@@ -376,12 +387,12 @@ void CHMFrame::OnRegisterExtension(wxCommandEvent& WXUNUSED(event))
 
 void CHMFrame::OnPrint(wxCommandEvent& WXUNUSED(event))
 {
-	wxString page = _html->GetOpenedPage();
-
-	if(page.IsEmpty())
-		_ep->PrintText(greeting);
-	else
-		_ep->PrintFile(page);
+        int sizes[7];
+	        for(int i = -3; i <= 3; ++i)
+	                sizes[i+3] = _fontSize + i * 2;
+			
+	_ep->SetFonts(_normalFont, _fixedFont, sizes);
+	_ep->PrintFile(_html->GetOpenedPage());
 }
 
 
@@ -520,7 +531,8 @@ bool CHMFrame::LoadCHM(const wxString& archive)
 {
 	wxBusyCursor bc;
 	wxLogNull wln;
-	bool rtn;
+	
+	bool rtn = false;
 	SaveBookmarks();
 	_nb->SetSelection(0);
 
@@ -547,7 +559,7 @@ bool CHMFrame::LoadCHM(const wxString& archive)
 
 	if(!rtn) { // Error, could not load CHM file
 		if(_tcl->GetCount())
-			_tcl->DeleteAllItems();
+			_tcl->DeleteChildren(_tcl->GetRootItem());
 		if(_sw->IsSplit()) {
 			_sw->Unsplit(_nb);
 			_nb->Show(FALSE);
@@ -595,6 +607,9 @@ void CHMFrame::UpdateCHMInfo()
 	if(!chmf)
 		return;
 
+	wxWindowDisabler wwd;
+	wxBusyInfo wait(_("Loading, please wait.."));
+
 	wxString filename = chmf->ArchiveName();
 	if(!filename.IsEmpty()) {		
 		_fh.AddFileToHistory(filename);
@@ -610,7 +625,8 @@ void CHMFrame::UpdateCHMInfo()
 	wxString title = chmf->Title();
 
 	if(_tcl->GetCount())
-		_tcl->DeleteAllItems();
+		_tcl->DeleteChildren(_tcl->GetRootItem());
+
 	chmf->GetTopicsTree(_tcl);
 	chmf->GetIndex(_cip->GetResultsList());
 	
@@ -633,6 +649,9 @@ void CHMFrame::UpdateCHMInfo()
 		long fs = -1;		
 		fontFace.BeforeLast(wxT(',')).AfterLast(wxT(',')).ToLong(&fs);
 
+		if(fs < 10)
+			fs = 10;
+
 		wxFont font((int)fs, wxDEFAULT, wxNORMAL, wxNORMAL, 
 			    FALSE, fontFace.BeforeFirst(wxT(',')), 
 			    chmf->DesiredEncoding());
@@ -677,7 +696,7 @@ void CHMFrame::UpdateCHMInfo()
 		noSpecialFont = true;
 	}
 #endif
-
+	
 	// if we have contents..
 	if(_tcl->GetCount() >= 1) {		
 		if(!_sw->IsSplit()) {
@@ -782,35 +801,18 @@ wxPanel* CHMFrame::CreateContentsPanel()
 	temp->SetAutoLayout(TRUE);
         temp->SetSizer(sizer);
 
-#ifndef __WXMSW__
-	wxImageList *tclBL = new wxImageList(16, 16);
-	tclBL->Add(wxIcon(hbook_closed_xpm));
-	tclBL->Add(wxIcon(hbook_closed_xpm));
-	tclBL->Add(wxIcon(hbook_open_xpm));
-	tclBL->Add(wxIcon(hbook_open_xpm));
-	wxImageList *tclIL = new wxImageList(16, 16);
-	tclIL->Add(wxIcon(hpage_xpm));
-#else
 	wxImageList *il = new wxImageList(16, 16);
 	il->Add(wxIcon(hbook_closed_xpm));
 	il->Add(wxIcon(hbook_open_xpm));
 	il->Add(wxIcon(hpage_xpm));
-#endif
 
 	_tcl = new wxTreeCtrl(temp, ID_TreeCtrl, wxDefaultPosition, 
 			      wxDefaultSize, 
-#ifndef __WXMSW__
-			      wxTR_HAS_BUTTONS |
-#endif
 			      wxSUNKEN_BORDER | wxTR_HIDE_ROOT
 			      | wxTR_LINES_AT_ROOT);
 
-#ifndef __WXMSW__
-	_tcl->AssignButtonsImageList(tclBL);
-	_tcl->AssignImageList(tclIL);
-#else
 	_tcl->AssignImageList(il);	
-#endif
+	_tcl->AddRoot(_("Topics"));
 
 	_cb = new wxComboBox(temp, ID_Bookmarks, wxT(""), wxDefaultPosition,
 			     wxDefaultSize, 0, NULL, wxCB_DROPDOWN 
