@@ -3,6 +3,8 @@
   Copyright (C) 2003  Razvan Cojocaru <razvanco@gmx.net>
   Mac OS specific patches contributed by Chanler White 
   <cawhite@nwrails.com>
+  "Save link as" patch contributed by Joerg Wunsch
+  <joerg_wunsch@users.sourceforge.net>
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,6 +32,8 @@
 #include <wx/clipbrd.h>
 #include <wx/filename.h>
 #include <wx/uri.h>
+#include <wx/wfstream.h>
+#include <memory>
 
 
 
@@ -42,6 +46,7 @@ CHMHtmlWindow::CHMHtmlWindow(wxWindow *parent, wxTreeCtrl *tc, CHMFrame *frame)
 	_menu->Append(ID_PopupForward, _("For&ward"));
 	_menu->Append(ID_PopupBack, _("&Back"));
 	_menu->Append(ID_CopyLink, _("Copy &link location"));
+	_menu->Append(ID_SaveLinkAs, _("&Save link as.."));
 
 	_menu->AppendSeparator();
 	_menu->Append(ID_CopySel, _("&Copy selection"));
@@ -70,17 +75,17 @@ bool CHMHtmlWindow::LoadPage(const wxString& location)
 	   !location.AfterLast(wxT('/')).IsEmpty() && 
 	   _tcl->GetCount() > 1) {
 
-		wxString srch;
-		if(tmp.StartsWith(wxT("/")) || tmp.StartsWith(wxT("file:")))
-			srch = tmp.AfterLast(wxT(':')).AfterFirst(
-				wxT('/')).BeforeFirst(wxT('#'));
-		else
-			srch = tmp.BeforeFirst(wxT('#'));
+		wxFileName fwfn(tmp.AfterLast(wxT(':')).BeforeFirst(wxT('#')), 
+				wxPATH_UNIX);
+
+		wxString cwd = GetParser()->GetFS()->
+			GetPath().AfterLast(wxT(':'));
+		fwfn.Normalize(wxPATH_NORM_DOTS | wxPATH_NORM_ABSOLUTE, cwd);
 		
 		// Sync will call SelectItem() on the tree item
 		// if it finds one, and that in turn will call
 		// LoadPage() with _syncTree set to false.
-		Sync(_tcl->GetRootItem(), srch);
+		Sync(_tcl->GetRootItem(), fwfn.GetFullPath(wxPATH_UNIX));
 
 		if(_found)
 			_found = false;
@@ -102,11 +107,8 @@ void CHMHtmlWindow::Sync(wxTreeItemId root, const wxString& page)
 
 	if(data)
 		url = (data->_url).BeforeFirst(wxT('#'));
-	
-	if(data && (!url.CmpNoCase(page) || 
-		    !url.CmpNoCase(GetPrefix(GetOpenedPage()) + wxT("/") 
-				   + page))) {
-		// Order counts!
+
+	if(data && !url.CmpNoCase(page)) {
 		_found = true;
 		_tcl->SelectItem(root);
 		return;
@@ -294,6 +296,48 @@ void CHMHtmlWindow::OnCopyLink(wxCommandEvent& WXUNUSED(event))
 	}
 }
 
+void CHMHtmlWindow::OnSaveLinkAs(wxCommandEvent& WXUNUSED(event))
+{
+	std::auto_ptr<wxFSFile> f(m_FS->OpenFile(_link));
+
+	if (f.get() == NULL) {
+		::wxMessageBox(_("OpenFile(") + _link + _(") failed"),
+			       _("Error"), wxOK, this);
+		return;
+	}
+
+	wxFileName wfn(_link);
+	wxString suggestedName = wfn.GetFullName();
+
+	if(suggestedName.IsEmpty())
+		suggestedName = wxT("index.html");
+	
+	wxString filename = ::wxFileSelector(_("Save as"),
+					     wxT(""),
+					     suggestedName,
+					     wxT(""),
+					     wxT("*.*"),
+					     wxSAVE | wxOVERWRITE_PROMPT,
+					     this);
+	if (!filename.empty()) {
+		wxInputStream *s = f->GetStream();
+		wxFileOutputStream out(filename);
+		if (!out.IsOk()) {
+			::wxMessageBox(_("Error creating file ") + filename,
+				       _("Error"), wxOK, this);
+		} else {
+			char buffer[4096];
+			while (!s->Eof()) {
+				s->Read(buffer, sizeof(buffer));
+				size_t nbytes = s->LastRead();
+				out.Write((void *)buffer, nbytes);
+			}
+			::wxMessageBox(_("Saved file ") + filename,
+				       _("Success"), wxOK, this);
+		}
+	}
+}
+
 
 void CHMHtmlWindow::OnRightClick(wxMouseEvent& event)
 {
@@ -303,6 +347,7 @@ void CHMHtmlWindow::OnRightClick(wxMouseEvent& event)
 	_menu->Enable(ID_PopupForward, HistoryCanForward());
 	_menu->Enable(ID_PopupBack, HistoryCanBack());
 	_menu->Enable(ID_CopyLink, false);
+	_menu->Enable(ID_SaveLinkAs, false);
 
         int x, y;
         CalcUnscrolledPosition(event.m_x, event.m_y, &x, &y);
@@ -318,6 +363,7 @@ void CHMHtmlWindow::OnRightClick(wxMouseEvent& event)
 	if(linfo) {
 		_link = linfo->GetHref();
 		_menu->Enable(ID_CopyLink, true);
+		_menu->Enable(ID_SaveLinkAs, true);
 	}
 
 	PopupMenu(_menu, event.GetPosition());
@@ -343,6 +389,7 @@ BEGIN_EVENT_TABLE(CHMHtmlWindow, wxHtmlWindow)
 	EVT_MENU(ID_PopupForward, CHMHtmlWindow::OnForward)
 	EVT_MENU(ID_PopupBack, CHMHtmlWindow::OnBack)
 	EVT_MENU(ID_CopyLink, CHMHtmlWindow::OnCopyLink)
+	EVT_MENU(ID_SaveLinkAs, CHMHtmlWindow::OnSaveLinkAs)
 	EVT_CHAR(CHMHtmlWindow::OnChar)
 	EVT_RIGHT_DOWN(CHMHtmlWindow::OnRightClick)
 	EVT_SIZE(CHMHtmlWindow::OnSize)
