@@ -136,8 +136,11 @@ size_t CHMInputStream::OnSysRead(void *buffer, size_t bufsize)
 	return bufsize;
 }
 
-
+#ifdef __WXMSW__
+wxFileOffset CHMInputStream::OnSysSeek(wxFileOffset seek, wxSeekMode mode)
+#else
 off_t CHMInputStream::OnSysSeek(off_t seek, wxSeekMode mode)
+#endif
 {
 	switch(mode) {
 	case wxFromCurrent:
@@ -175,3 +178,17 @@ bool CHMInputStream::Init(const wxString& archive)
 	return true;
 }
 
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
