@@ -82,13 +82,21 @@ protected:
 	  \param mode Seek from the beginning, current position, etc.
 	  \return Position in the file.
 	 */
+#ifdef __WXMSW__
+	virtual wxFileOffset OnSysSeek(wxFileOffset seek, wxSeekMode mode);
+#else
 	virtual off_t OnSysSeek(off_t seek, wxSeekMode mode);
+#endif
 
 	/*!
 	  \brief Asks what is the current position in the file.
 	  \return Current position.
 	 */
+#ifdef __WXMSW__
+	virtual wxFileOffset OnSysTell() const { return _currPos; }
+#else
 	virtual off_t OnSysTell() const { return _currPos; }
+#endif
 
 private:
 	//! Helper. Inits the cache.
@@ -104,3 +112,16 @@ private:
 
 #endif // __CHMINPUTSTREAM_H_
 
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
