@@ -119,3 +119,17 @@ private:
 
 #endif // __CHMSEARCHPANEL_HPP_
 
+
+/*
+  Local Variables:
+  mode: c++
+  c-basic-offset: 8
+  tab-width: 8
+  c-indent-comments-syntactically-p: t
+  c-tab-always-indent: t
+  indent-tabs-mode: t
+  End:
+*/
+
+// vim:shiftwidth=8:autoindent:tabstop=8:noexpandtab:softtabstop=8
+
