@@ -151,7 +151,7 @@ HHCParser::HHCParser(wxFontEncoding enc, wxTreeCtrl *tree, CHMListCtrl *list)
 	memset(_parents, 0, TREE_BUF_SIZE*sizeof(wxTreeItemId));
 	
 	if(_tree)
-		_parents[_level] = _tree->AddRoot(_("Topics"));
+		_parents[_level] = _tree->GetRootItem();
 }
 
 
@@ -196,7 +196,8 @@ void HHCParser::handleTag(const std::string& tag)
 	++_counter;
 
 	if((_counter % TIME_TO_YIELD) == 0) {
-		wxSafeYield();	
+		//wxSafeYield();
+		wxYield();
 		_counter = 0;
 	}
 
@@ -224,20 +225,23 @@ void HHCParser::handleTag(const std::string& tag)
 		if(tagName == "/object") {
 			_inobject = false;
 
+			if(!_value.empty() && _value[0] != '/')
+				_value = std::string("/") + _value;
+
 			wxString name = CURRENT_CHAR_STRING(_name.c_str());
 			wxString value = CURRENT_CHAR_STRING(_value.c_str());
 
 			if(_htmlChars) {
 				name = replaceHTMLChars(name);
+				value = replaceHTMLChars(value);
 				_htmlChars = false;
 			}
- 
-			name = translateEncoding(name);
+
+			name = translateEncoding(name, _enc, _cv);
 
 			addToTree(name, value);
 			addToList(name, value);
 
-
 		} else if(tagName == "param") {
 
 			std::string name, value;
@@ -246,10 +250,10 @@ void HHCParser::handleTag(const std::string& tag)
 			
 			if(name == "name" && _name.empty()) {
 				_name = value;
-				_htmlChars = special;
-
+				_htmlChars = _htmlChars || special;
 			} else if(name == "local" && _value.empty()) {
 				_value = value;
+				_htmlChars = _htmlChars || special;
 			}
 		}
 	
@@ -353,70 +357,31 @@ bool HHCParser::getParameters(const char* input, std::string& name,
 }
 
 
-inline
-wxString HHCParser::translateEncoding(const wxString& input)
-{
-	if(input.IsEmpty())
-		return wxEmptyString;
-
-#if wxUSE_UNICODE
-#	define BUF_SIZE 1024
-
-	if(_enc != wxFONTENCODING_SYSTEM) {
-		wchar_t buf2[BUF_SIZE];
-		size_t len = (input.Length() < BUF_SIZE) ?
-			input.Length() : BUF_SIZE;
-
-		size_t ret = _cv.MB2WC(buf2, input.mb_str(), len);
-
-		if(ret) { 
-			if(ret == (size_t)(-1)) // morons
-				return wxString(buf2, input.Length());
-
-			return wxString(buf2, ret);
-		} else
-			return wxEmptyString;
-	}
-#endif
-	return input;
-}
-
-
 void HHCParser::addToTree(const wxString& name, const wxString& value)
 {
 	if(!_tree)
 		return;
 
 	if(!name.IsEmpty()) {
-		
+
 		int parentIndex = _level ? _level - 1 : 0;
 
 		_parents[_level] = 
-#ifndef __WXMSW__
-			_tree->AppendItem(_parents[parentIndex], name, 0, 0,
-					  new URLTreeItem(value));
-#else
 			_tree->AppendItem(_parents[parentIndex], name, 2, 2,
 					  new URLTreeItem(value));
-#endif
 		if(!_level)
 			_parents[0] = _tree->GetRootItem();
 		else {
-#ifndef __WXMSW__
-			_tree->SetItemImage(_parents[parentIndex], -1,
-					    wxTreeItemIcon_Normal);
-			
-			_tree->SetItemImage(_parents[parentIndex], -1, 
-					    wxTreeItemIcon_Selected);
-#else
 			if(_tree->GetItemImage(_parents[parentIndex]) != 0) {
 				_tree->SetItemImage(_parents[parentIndex], 0,
 						wxTreeItemIcon_Normal);
 
+				_tree->SetItemImage(_parents[parentIndex], 0,
+						wxTreeItemIcon_Selected);
+
 				_tree->SetItemImage(_parents[parentIndex], 1,
 						wxTreeItemIcon_Expanded);
 			}	
-#endif
 		}
 	}
 }
@@ -455,9 +420,10 @@ wxString HHCParser::replaceHTMLChars(const wxString& input)
 				inSpecial = false;
 
 				unsigned code = getHTMLCode(special);
-				if(code)
-					result.Append(charForCode(code));
 
+				if(code)
+					result.Append(charForCode(code, _cv,
+								  false));
 				continue;
 			}
 		}
@@ -482,6 +448,7 @@ unsigned HHCParser::getHTMLCode(const wxString& name)
 					  substitutions_cnt,
 					  sizeof(HTMLChar),
 					  HTMLCharCompare);	
+
 	if(hc)
 		return hc->code;
 
@@ -489,33 +456,6 @@ unsigned HHCParser::getHTMLCode(const wxString& name)
 }
 
 
-inline
-wxChar HHCParser::charForCode(unsigned code)
-{
-#if !wxUSE_UNICODE
-
-#	if wxUSE_WCHAR_T
-
-	char buf[2];
-	wchar_t wbuf[2];
-	wbuf[0] = (wchar_t)code;
-	wbuf[1] = 0;
-
-	if (_cv.WC2MB(buf, wbuf, 2) == (size_t)-1)
-		return '?';
-
-	return buf[0];
-
-#	else
-	return (code < 256) ? (wxChar)code : '?';
-
-#	endif                         
-
-#else
-	return (wxChar)code;
-#endif
-}
-
 
 /*
   Local Variables:
